##
# File: PdbxAtomSite.py
# Date: 21-Mar-2011  Jdw
#
# Updated:
#  23-Oct-2012 jdw  Update path and reorganize
###
"""
A collection of classes to assemble/merge atom site records.

Methods are provided to integrated separately stored anisotropic temperature
factor tensor components into atom site records.

"""
__docformat__ = "restructuredtext en"
__author__    = "John Westbrook"
__email__     = "john.westbrook@rcsb.org"
__license__   = "Apache 2.0"
__version__   = "V0.01"

import sys
from mmcif.api.PdbxContainers import *
from mmcif.api.DataCategory   import DataCategory

class PdbxAtomSite(object):
    """
    """
    def __init__(self,verbose=True,log=sys.stdout):
        self.__verbose=verbose
        self.__debug=False
        self.__lfh=log
        #
        self.__dBlock=None

        self.__atomSiteTags=['PDB_RECORD','ATOM_ID','AUTH_ATOM_ID','TYPE_SYMBOL','AUTH_COMP_ID', 'AUTH_ASYM_ID','AUTH_SEQ_ID', 'PDB_INS_CODE',
                             'CARTN_X','CARTN_Y','CARTN_Z', 'FORMAL_CHARGE','MODEL_NUMBER', 'GROUP_ID', 'OCCUPANCY','B_ISO', 'ALT_ID',
                             'U11', 'U22', 'U33', 'U12', 'U13', 'U23']

        self.__atomSiteMap={
            'PDB_RECORD'    : '_atom_site.pdbx_group_PDB',
            'ATOM_ID'       : '_atom_site.id',
            'AUTH_ATOM_ID'  : '_atom_site.auth_atom_id',
            'TYPE_SYMBOL'   : '_atom_site.type_symbol',
            'AUTH_COMP_ID'  : '_atom_site.auth_comp_id',
            'AUTH_ASYM_ID'  : '_atom_site.auth_asym_id',
            'AUTH_SEQ_ID'   : '_atom_site.auth_seq_id',
            'PDB_INS_CODE'  : '_atom_site.pdbx_PDB_ins_code',
            'CARTN_X'       : '_atom_site.Cartn_x',
            'CARTN_Y'       : '_atom_site.Cartn_y',
            'CARTN_Z'       : '_atom_site.Cartn_z',
            'FORMAL_CHARGE' : '_atom_site.pdbx_formal_charge',
            'MODEL_NUMBER'  : '_atom_site.pdbx_PDB_model_num',
            'GROUP_ID'      : '_atom_site.pdbx_group_id',
            'OCCUPANCY'     : '_atom_site.occupancy',
            'B_ISO'         : '_atom_site.B_iso_or_equiv',
            'ALT_ID'        : '_atom_site.label_alt_id',
            'U11'           : '_atom_site.aniso_U[1][1]',
            'U22'           : '_atom_site.aniso_U[2][2]',
            'U33'           : '_atom_site.aniso_U[3][3]',
            'U12'           : '_atom_site.aniso_U[1][2]',
            'U13'           : '_atom_site.aniso_U[1][3]',
            'U23'           : '_atom_site.aniso_U[2][3]'
            }

        self.__atomSiteAnisoTags =['PDB_RECORD','ATOM_ID','U11','U22','U33','U12','U13','U23']
        self.__atomSiteAnisoMap = {
            'PDB_RECORD' : '_atom_site_anisotrop.group_PDB',
            'ATOM_ID'    : '_atom_site_anisotrop.id',
            'U11'        : '_atom_site_anisotrop.U[1][1]',
            'U22'        : '_atom_site_anisotrop.U[2][2]',
            'U33'        : '_atom_site_anisotrop.U[3][3]',
            'U12'        : '_atom_site_anisotrop.U[1][2]',
            'U13'        : '_atom_site_anisotrop.U[1][3]',
            'U23'        : '_atom_site_anisotrop.U[2][3]'}


        """
        Example PDBx atom_site and atom_site_aniso data sections from pdb 1g2r

loop_
_atom_site.group_PDB
_atom_site.id
_atom_site.type_symbol
_atom_site.label_atom_id
_atom_site.label_alt_id
_atom_site.label_comp_id
_atom_site.label_asym_id
_atom_site.label_entity_id
_atom_site.label_seq_id
_atom_site.pdbx_PDB_ins_code
_atom_site.Cartn_x
_atom_site.Cartn_y
_atom_site.Cartn_z
_atom_site.occupancy
_atom_site.B_iso_or_equiv
_atom_site.Cartn_x_esd
_atom_site.Cartn_y_esd
_atom_site.Cartn_z_esd
_atom_site.occupancy_esd
_atom_site.B_iso_or_equiv_esd
_atom_site.pdbx_formal_charge
_atom_site.auth_seq_id
_atom_site.auth_comp_id
_atom_site.auth_asym_id
_atom_site.auth_atom_id
_atom_site.pdbx_PDB_model_num
ATOM   1   N N   . ARG A 1 7   ? 16.757  8.703   13.450  0.62 21.74 ? ? ? ? ? ? 4   ARG A N   1
ATOM   2   C CA  . ARG A 1 7   ? 17.014  7.699   12.351  0.62 18.89 ? ? ? ? ? ? 4   ARG A CA  1
ATOM   3   C C   . ARG A 1 7   ? 16.929  8.224   10.937  0.62 18.59 ? ? ? ? ? ? 4   ARG A C   1
# ...

#
loop_
_atom_site_anisotrop.id
_atom_site_anisotrop.type_symbol
_atom_site_anisotrop.pdbx_label_atom_id
_atom_site_anisotrop.pdbx_label_alt_id
_atom_site_anisotrop.pdbx_label_comp_id
_atom_site_anisotrop.pdbx_label_asym_id
_atom_site_anisotrop.pdbx_label_seq_id
_atom_site_anisotrop.U[1][1]
_atom_site_anisotrop.U[2][2]
_atom_site_anisotrop.U[3][3]
_atom_site_anisotrop.U[1][2]
_atom_site_anisotrop.U[1][3]
_atom_site_anisotrop.U[2][3]
_atom_site_anisotrop.U[1][1]_esd
_atom_site_anisotrop.U[2][2]_esd
_atom_site_anisotrop.U[3][3]_esd
_atom_site_anisotrop.U[1][2]_esd
_atom_site_anisotrop.U[1][3]_esd
_atom_site_anisotrop.U[2][3]_esd
_atom_site_anisotrop.pdbx_auth_seq_id
_atom_site_anisotrop.pdbx_auth_comp_id
_atom_site_anisotrop.pdbx_auth_asym_id
_atom_site_anisotrop.pdbx_auth_atom_id
1   N N   . ARG A 7   0.3083 0.2627 0.2550 0.0385  0.0117  -0.0184 ? ? ? ? ? ? 4   ARG A N
2   C CA  . ARG A 7   0.2277 0.2454 0.2447 0.0416  0.0180  -0.0114 ? ? ? ? ? ? 4   ARG A CA
3   C C   . ARG A 7   0.2320 0.2201 0.2544 0.0298  0.0127  -0.0050 ? ? ? ? ? ? 4   ARG A C
4   O O   . ARG A 7   0.1796 0.1693 0.2865 0.0514  0.0604   0.0124  ? ? ? ? ? ? 4   ARG A O
5   C CB  . ARG A 7   0.2434 0.2587 0.2580 0.0271  0.0224  -0.0195  ? ? ? ? ? ? 4   ARG A CB
6   C CG  . ARG A 7   0.2760 0.3058 0.2823 0.0220  0.0253   0.0067  ? ? ? ? ? ? 4   ARG A CG
7   C CD  . ARG A 7   0.2900 0.2967 0.3303 0.0019  0.0299  -0.0281  ? ? ? ? ? ? 4   ARG A CD
8   N NE  . ARG A 7   0.2469 0.2965 0.3095 0.0233  0.0535  -0.0339  ? ? ? ? ? ? 4   ARG A NE
# ....
        """


        self.__clear();

    def __clear(self):
        self.__uD={}



    def __categoryPart(self, name):
        tname=""
        if name.startswith("_"):
            tname=name[1:]
        else:
            tname=name

        i = tname.find(".")
        if i == -1:
            return tname
        else:
            return tname[:i]

    def __attributePart(self, name):
        i = name.find(".")
        if i == -1:
            return None
        else:
            return name[i+1:]

    def setAtomSiteBlock(self,block=None):
        self.__clear()
        return self.__setDataBlock(block)


    def __setDataBlock(self,dataBlock=None):
        """
        """
        ok=False
        try:
            if (dataBlock.getType() == 'data'):
                self.__dBlock=dataBlock
                ok=True
            else:
                self.__dBlock=None
        except Exception as e:
            logger.exception("Failing with %s" % str(e))
            pass

        return ok

    def getAnisoTensorData(self,catName='atom_site_anisotrop'):
        """ Return a dictionary of lists of anisotropic tensor elements stored in the
            in the input category (i.e. atom_site_anisotrop') in a dictionary with
            an atom serial number key (i.e. ATOM_ID).

        """
        catObj=self.__dBlock.getObj(catName)
        nRows=catObj.getRowCount()

        itDict={}
        itNameList=catObj.getItemNameList()
        for idxIt,itName in enumerate(itNameList):
            itDict[itName]=idxIt
        #
        # column name to row index map -
        cM={}
        for k,v in self.__atomSiteAnisoMap.items():
            if itDict.has_key(v):
                cM[k]=itDict[v]
            else:
                cM[k]=-1
        #
        self.__clear()
        #
        self.__uD={}
        rowList=catObj.getRowList()
        for row in rowList:
            if ( (row[cM['ATOM_ID']] is not None) and (row[cM['U11']] is not None)  ):
                self.__uD[row[cM['ATOM_ID']]] = (row[cM['U11']], row[cM['U22']], row[cM['U33']], row[cM['U12']], row[cM['U13']], row[cM['U23']])

        return self.__uD


    def mergeAnisoTensorData(self,inCatName='atom_site', outCatName='atom_site_xray_aniso'):
        """ Read the atom records in input category (i.e. default atom_site) and integrate the
            anisotropic tensor elements in the internal dictionary (self.__uD) into each record.

            The integrated records are written to a new category (i.e. outCatName='atom_site_xray_aniso')
            in the current datablock
        """
        catObjIn=self.__dBlock.getObj(inCatName)
        nRows=catObjIn.getRowCount()

        # make out category container --
        catObjOut=DataCategory(outCatName)
        for tag in self.__atomSiteTags:
            att = self.__attributePart(self.__atomSiteMap[tag])
            catObjOut.appendAttribute(att)

        itDict={}
        itNameList=catObjIn.getItemNameList()
        for idxIt,itName in enumerate(itNameList):
            itDict[itName]=idxIt
        #
        # column name to row index map  or -1
        cM={}
        for k,v in self.__atomSiteMap.items():
            if itDict.has_key(v):
                cM[k]=itDict[v]
            else:
                cM[k]=-1
        #
        #self.__clear()
        #

        rowListIn=catObjIn.getRowList()
        for row in rowListIn:
            # copy corresponding data items --
            #
            rD={}
            for k in cM.keys():
                rD[k]=row[cM[k]]
            #
            #logger.info("KEYS %r\n\n" % self.__uD.keys())
            #logger.info("ATOM_ID %r\n" % rD['ATOM_ID'])

            if self.__uD.has_key(rD['ATOM_ID']):
                tup = self.__uD[rD['ATOM_ID']]
                for i,k in enumerate(self.__atomSiteAnisoTags[2:]):
                    rD[k]=tup[i]
            #
            newR=[]
            for k in self.__atomSiteTags:
                if rD.has_key(k):
                    newR.append(rD[k])
                else:
                    newR.append('?')

            catObjOut.append(newR)
        #
        self.__dBlock.append(catObjOut)
        return catObjOut








##
# File: PdbxMessageInfoCategoryStyle.py
# Date: 6-Aug-2013  John Westbrook
#
# Updates:
#  2013-08-13   RPS  corrected typos in category name identifiers
#
#  15-Aug-2013  jdw  change read_status to send_status in  'pdbx_deposition_message_info'
#  15-Aug-2013  jdw  add category 'pdbx_deposition_message_status'
#  15-Aug-2013  RPS  changed 'read_status' to 'send_status' in 'pdbx_deposition_message_info'
#  30-Oct-2013  rps  added _pdbx_deposition_message_status.action_reqd to support UI feature
#                    for classifying messages (e.g. "action required", "unread")
#  29-Jul-2014  rps  added '_pdbx_deposition_message_file_reference.upload_file_name' to support
#                    multiple file references for a single message (i.e. so that original name can be
#                    used to more easily distinguish one file reference from another).
#  04-Dec-2014  rps  Added category, pdbx_deposition_message_origcomm_reference.
#                    Added item, pdbx_deposition_message_status.for_release.
#   5-Mar-2018  jdw Py2-Py3 and refactor for Python Packaging
##
"""
Standard style details for depositor message data.

"""
from __future__ import absolute_import
__docformat__ = "restructuredtext en"
__author__ = "John Westbrook"
__email__ = "john.westbrook@rcsb.org"
__license__ = "Apache 2.0"
__version__ = "V0.01"

from mmcif_utils.style.PdbxCategoryStyleBase import PdbxCategoryStyleBase


class PdbxMessageCategoryStyle(PdbxCategoryStyleBase):
    #
    _styleId = "PDBX_DEPOSITION_MESSAGE_INFO_V1"

    _categoryInfo = [('pdbx_deposition_message_info', 'table'),
                     ('pdbx_deposition_message_data', 'table'),
                     ('pdbx_deposition_message_file_reference', 'table'),
                     ('pdbx_deposition_message_origcomm_reference', 'table'),
                     ('pdbx_deposition_message_status', 'table')
                     ]

    _cDict = {
        'pdbx_deposition_message_info': [
            ('_pdbx_deposition_message_info.ordinal_id', '%s', 'str', ''),
            ('_pdbx_deposition_message_info.message_id', '%s', 'str', ''),
            ('_pdbx_deposition_message_info.deposition_data_set_id', '%s', 'str', ''),
            ('_pdbx_deposition_message_info.timestamp', '%s', 'str', ''),
            ('_pdbx_deposition_message_info.sender', '%s', 'str', ''),
            ('_pdbx_deposition_message_info.context_type', '%s', 'str', ''),
            ('_pdbx_deposition_message_info.context_value', '%s', 'str', ''),
            ('_pdbx_deposition_message_info.parent_message_id', '%s', 'str', ''),
            ('_pdbx_deposition_message_info.message_subject', '%s', 'str', ''),
            ('_pdbx_deposition_message_info.message_text', '%s', 'str', ''),
            ('_pdbx_deposition_message_info.message_type', '%s', 'str', ''),
            ('_pdbx_deposition_message_info.send_status', '%s', 'str', '')
        ],
        'pdbx_deposition_message_data': [
            ('_pdbx_deposition_message_data.ordinal_id', '%s', 'str', ''),
            ('_pdbx_deposition_message_data.message_id', '%s', 'str', ''),
            ('_pdbx_deposition_message_data.deposition_data_set_id', '%s', 'str', ''),
            ('_pdbx_deposition_message_data.message_type', '%s', 'str', ''),
            ('_pdbx_deposition_message_data.message_subject', '%s', 'str', ''),
            ('_pdbx_deposition_message_data.message_text', '%s', 'str', '')
        ],
        'pdbx_deposition_message_file_reference': [
            ('_pdbx_deposition_message_file_reference.ordinal_id', '%s', 'str', ''),
            ('_pdbx_deposition_message_file_reference.message_id', '%s', 'str', ''),
            ('_pdbx_deposition_message_file_reference.deposition_data_set_id', '%s', 'str', ''),
            ('_pdbx_deposition_message_file_reference.content_type', '%s', 'str', ''),
            ('_pdbx_deposition_message_file_reference.content_format', '%s', 'str', ''),
            ('_pdbx_deposition_message_file_reference.partition_number', '%s', 'str', ''),
            ('_pdbx_deposition_message_file_reference.version_id', '%s', 'str', ''),
            ('_pdbx_deposition_message_file_reference.storage_type', '%s', 'str', ''),
            ('_pdbx_deposition_message_file_reference.upload_file_name', '%s', 'str', '')
        ],
        'pdbx_deposition_message_origcomm_reference': [
            ('_pdbx_deposition_message_origcomm_reference.ordinal_id', '%s', 'str', ''),
            ('_pdbx_deposition_message_origcomm_reference.message_id', '%s', 'str', ''),
            ('_pdbx_deposition_message_origcomm_reference.deposition_data_set_id', '%s', 'str', ''),
            ('_pdbx_deposition_message_origcomm_reference.orig_message_id', '%s', 'str', ''),
            ('_pdbx_deposition_message_origcomm_reference.orig_deposition_data_set_id', '%s', 'str', ''),
            ('_pdbx_deposition_message_origcomm_reference.orig_timestamp', '%s', 'str', ''),
            ('_pdbx_deposition_message_origcomm_reference.orig_sender', '%s', 'str', ''),
            ('_pdbx_deposition_message_origcomm_reference.orig_recipient', '%s', 'str', ''),
            ('_pdbx_deposition_message_origcomm_reference.orig_message_subject', '%s', 'str', ''),
            ('_pdbx_deposition_message_origcomm_reference.orig_attachments', '%s', 'str', '')
        ],
        'pdbx_deposition_message_status': [
            ('_pdbx_deposition_message_status.message_id', '%s', 'str', ''),
            ('_pdbx_deposition_message_status.deposition_data_set_id', '%s', 'str', ''),
            ('_pdbx_deposition_message_status.read_status', '%s', 'str', ''),
            ('_pdbx_deposition_message_status.action_reqd', '%s', 'str', ''),
            ('_pdbx_deposition_message_status.for_release', '%s', 'str', '')
        ]
    }
    _suppressList = []
    _excludeList = []
    #

    def __init__(self):
        super(PdbxMessageCategoryStyle, self).__init__(styleId=PdbxMessageCategoryStyle._styleId,
                                                       catFormatL=PdbxMessageCategoryStyle._categoryInfo,
                                                       catItemD=PdbxMessageCategoryStyle._cDict,
                                                       excludeList=PdbxMessageCategoryStyle._excludeList,
                                                       suppressList=PdbxMessageCategoryStyle._suppressList)



# File: setup.py
# Date: 9-Mar-2018
#
# Update:
#
import re
from setuptools import setup, find_packages


packages = []
thisPackage = 'mmcif.utils'
requires = ['future', 'six', 'mmcif']


with open('mmcif_utils/__init__.py', 'r') as fd:
    version = re.search(r'^__version__\s*=\s*[\'"]([^\'"]*)[\'"]',
                        fd.read(), re.MULTILINE).group(1)

if not version:
    raise RuntimeError('Cannot find version information')

setup(
    name=thisPackage,
    version=version,
    description='mmCIF Utility Classes',
    long_description="See:  README.md",
    author='John Westbrook',
    author_email='john.westbrook@rcsb.org',
    url='http://mmcif.wwpdb.org',
    #
    license='Apache 2.0',
    classifiers=(
        'Development Status :: 3 - Alpha',
        # 'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'Natural Language :: English',
        'License :: OSI Approved :: Apache Software License',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
    ),
    # entry_points={
    #    'console_scripts': [
    #        'onedep_validate_cli=onedep.cli.validate_cli:run',
    #    ]
    # },
    #
    install_requires=['future', 'six', 'mmcif'],
    packages=find_packages(exclude=['mmcif_utils.tests', 'tests.*']),
    package_data={
        # If any package contains *.md or *.rst ...  files, include them:
        '': ['*.md', '*.rst', "*.txt"],
    },
    #

    #
    test_suite="mmcif_utils.tests",
    tests_require=['tox'],
    #
    # Not configured ...
    extras_require={
        'dev': ['check-manifest'],
        'test': ['coverage'],
    },
    # Added for
    command_options={
        'build_sphinx': {
            'project': ('setup.py', thisPackage),
            'version': ('setup.py', version),
            'release': ('setup.py', version)
        }
    },
    zip_safe=True,
)



##
# File: ChemCompCategoryModelStyle.py
# Date: 2-Nov-2014  John Westbrook
#
# Updates:
#   5-Mar-2018  jdw Py2-Py3 and refactor for Python Packaging
##
"""
Standard style details for chemical component model data categories describing
 additional coordinate models for chemical components.
"""

from __future__ import absolute_import
__docformat__ = "restructuredtext en"
__author__ = "John Westbrook"
__email__ = "john.westbrook@rcsb.org"
__license__ = "Apache 2.0"
__version__ = "V0.01"


from mmcif_utils.style.PdbxCategoryStyleBase import PdbxCategoryStyleBase


class ChemCompModelCategoryStyle(PdbxCategoryStyleBase):
    #
    _styleId = "CHEM_COMP_MODEL_V1"

    _categoryInfo = [('pdbx_chem_comp_model', 'key-value'),
                     ('pdbx_chem_comp_model_reference', 'table'),
                     ('pdbx_chem_comp_model_feature', 'table'),
                     ('pdbx_chem_comp_model_atom', 'table'),
                     ('pdbx_chem_comp_model_bond', 'table'),
                     #('pdbx_chem_comp_model_identifier',    'table'),
                     ('pdbx_chem_comp_model_descriptor', 'table'),
                     ('pdbx_chem_comp_model_audit', 'table'),
                     ]
    _cDict = {
        'pdbx_chem_comp_model': [
            ('_pdbx_chem_comp_model.id', '%s', 'str', ''),
            ('_pdbx_chem_comp_model.comp_id', '%s', 'str', ''),
        ],
        'pdbx_chem_comp_model_atom': [
            ('_pdbx_chem_comp_model_atom.model_id', '%s', 'str', ''),
            ('_pdbx_chem_comp_model_atom.atom_id', '%s', 'str', ''),
            ('_pdbx_chem_comp_model_atom.type_symbol', '%s', 'str', ''),
            ('_pdbx_chem_comp_model_atom.charge', '%s', 'str', ''),
            ('_pdbx_chem_comp_model_atom.model_Cartn_x', '%s', 'str', ''),
            ('_pdbx_chem_comp_model_atom.model_Cartn_y', '%s', 'str', ''),
            ('_pdbx_chem_comp_model_atom.model_Cartn_z', '%s', 'str', ''),
            ('_pdbx_chem_comp_model_atom.ordinal_id', '%s', 'str', '')
        ],
        'pdbx_chem_comp_model_bond': [
            ('_pdbx_chem_comp_model_bond.model_id', '%s', 'str', ''),
            ('_pdbx_chem_comp_model_bond.atom_id_1', '%s', 'str', ''),
            ('_pdbx_chem_comp_model_bond.atom_id_2', '%s', 'str', ''),
            ('_pdbx_chem_comp_model_bond.value_order', '%s', 'str', ''),
            ('_pdbx_chem_comp_model_bond.ordinal_id', '%s', 'str', '')
        ],
        'pdbx_chem_comp_model_descriptor': [
            ('_pdbx_chem_comp_model_descriptor.model_id', '%s', 'str', ''),
            ('_pdbx_chem_comp_model_descriptor.type', '%s', 'str', ''),
            #('_pdbx_chem_comp_model_descriptor.program','%s','str',''),
            #('_pdbx_chem_comp_model_descriptor.program_version','%s','str',''),
            ('_pdbx_chem_comp_model_descriptor.descriptor', '%s', 'str', '')
        ],
        'pdbx_chem_comp_model_identifier': [
            ('_pdbx_chem_comp_model_identifier.model_id', '%s', 'str', ''),
            ('_pdbx_chem_comp_model_identifier.type', '%s', 'str', ''),
            #('_pdbx_chem_comp_model_identifier.program',        '%s','str',''),
            #('_pdbx_chem_comp_model_identifier.program_version','%s','str',''),
            ('_pdbx_chem_comp_model_identifier.identifier', '%s', 'str', '')
        ],
        'pdbx_chem_comp_model_audit': [
            ('_pdbx_chem_comp_model_audit.model_id', '%s', 'str', ''),
            ('_pdbx_chem_comp_model_audit.action_type', '%s', 'str', ''),
            ('_pdbx_chem_comp_model_audit.date', '%s', 'str', '')
            # ('_pdbx_chem_comp_model_audit.processing_site', '%s', 'str', ''),
            # ('_pdbx_chem_comp_model_audit.annotator', '%s', 'str', ''),
            # ('_pdbx_chem_comp_model_audit.details', '%s', 'str', '')
        ],
        'pdbx_chem_comp_model_reference': [
            ('_pdbx_chem_comp_model_reference.model_id', '%s', 'str', ''),
            ('_pdbx_chem_comp_model_reference.db_name', '%s', 'str', ''),
            ('_pdbx_chem_comp_model_reference.db_code', '%s', 'str', ''),
        ],

        'pdbx_chem_comp_model_feature': [
            ('_pdbx_chem_comp_model_reference.model_id', '%s', 'str', ''),
            ('_pdbx_chem_comp_model_reference.feature_name', '%s', 'str', ''),
            ('_pdbx_chem_comp_model_reference.feature_value', '%s', 'str', ''),
        ],

    }

    _suppressList = ['_pdbx_chem_comp_model_audit.annotator', '_pdbx_chem_comp_model_audit.details']
    _excludeList = ['_pdbx_chem_comp_model_audit.annotator', '_pdbx_chem_comp_model_audit.details']
    #

    def __init__(self):
        super(ChemCompModelCategoryStyle, self).__init__(styleId=ChemCompModelCategoryStyle._styleId,
                                                         catFormatL=ChemCompModelCategoryStyle._categoryInfo,
                                                         catItemD=ChemCompModelCategoryStyle._cDict,
                                                         excludeList=ChemCompModelCategoryStyle._excludeList,
                                                         suppressList=ChemCompModelCategoryStyle._suppressList)



##
# File: PdbxCategoryStyleBase.py
# Date: 28-Oct-2012  John Westbrook
#
# Update:
#  4-Nov-2012  jdw  Convert data structure to Functional api for style information.
#                   Provide methods for excluded and suppressed items.
# 11-Feb-2013  jdw  Add style identifier method.
#  5-Mar-2018  jdw Py2-Py3 and refactor for Python Packaging
##
"""
Base functions for accessing presentation style details for PDBx categories.

"""
from __future__ import absolute_import
from __future__ import print_function
__docformat__ = "restructuredtext en"
__author__ = "John Westbrook"
__email__ = "john.westbrook@rcsb.org"
__license__ = "Apache 2.0"
__version__ = "V0.01"


from mmcif.api.PdbxContainers import CifName


class PdbxCategoryStyleBase(object):

    ''' Base class for essential details for standardizing the presentation style of
        PDBx category data.

        styleId:

        __styleId

        catFormaL:

        __cL=[('<categoryName_1', 'style{table|key-value}', ('<categoryName_2>', 'style'),... ]

        catItemD:

        __cD={ <categoryName_1> : [ (itemName_1,feature1, feature2, ... ),(itemName_2,feature1, feature2, ... ), ... ],
              <categoryName_2> : [ (itemName_1,feature1, feature2, ... ),(itemName_2,feature1, feature2, ... ), ... ]
            }

         excludeList:
                      items separately flagged as candidates for exclusion -
         __excludeList-[itemName_1,itemName_2,... ]

         suppressList:
                              items separately flagged as candidates for value suppression-
         __suppressList=[itemname_1, itemName_2, ...]

    '''

    def __init__(self, styleId=None, catFormatL=None, catItemD=None, excludeList=None, suppressList=None):

        self.__styleId = styleId
        self.__cD = catItemD if catItemD is not None else {}
        self.__cL = catFormatL if catFormatL is not None else []
        self.__excludeList = excludeList if excludeList is not None else []
        self.__suppressList = suppressList if suppressList is not None else []
        #
        self.__indexExclude = {}
        for item in self.__excludeList:
            categoryName = CifName.categoryPart(item)
            if categoryName not in self.__indexExclude:
                self.__indexExclude[categoryName] = []
            self.__indexExclude[categoryName].append(CifName.attributePart(item))
        #
        self.__indexSuppress = {}
        for item in self.__suppressList:
            categoryName = CifName.categoryPart(item)
            if categoryName not in self.__indexSuppress:
                self.__indexSuppress[categoryName] = []
            self.__indexSuppress[categoryName].append(CifName.attributePart(item))
        #

    def getStyleId(self):
        """  Return an identifier for this style definition -
        """
        return self.__styleId

    def getExcludedList(self, categoryName):
        """ Return the list of excluded attributes or an empty list
        """
        try:
            return self.__indexExclude[categoryName]
        except Exception as e:
            return []

    def getSuppressList(self, categoryName):
        """ Return the list of suppress attributes or an empty list
        """
        try:
            return self.__indexSuppress[categoryName]
        except Exception as e:
            return []

    def checkStyleDefinition(self, ofh):
        """ Report internal consistency issues with the current style definition.

            Return True if ok or False otherwise

        """
        # category consistency.
        #
        missing = 0
        cDefList = list(self.__cD.keys())
        cFormatList = []
        for (categoryName, st) in self.__cL:
            cFormatList.append(categoryName)
            if categoryName not in cDefList:
                ofh.write("+WARN PdbxCategoryStyleBase.checkStyleDefinition - missing category in dictionary %s\n" % categoryName)
                missing += 1
        for categoryName in cDefList:
            if categoryName not in cFormatList:
                ofh.write("+WARN PdbxCategoryStyleBase.checkStyleDefinition - missing category in format list %s\n" % categoryName)
                missing += 1
        #
        for itemName in self.__excludeList:
            categoryName = CifName.categoryPart(itemName)
            if ((categoryName not in cFormatList) or (categoryName not in cDefList)):
                ofh.write("+WARN PdbxCategoryStyleBase.checkStyleDefinition - missing excluded category %s\n" % categoryName)
                missing += 1

            if itemName not in [itTup[0] for itTup in self.__cD[categoryName]]:
                ofh.write("+WARN PdbxCategoryStyleBase.checkStyleDefinition - missing excluded item %s\n" % itemName)
                missing += 1

        for itemName in self.__suppressList:
            categoryName = CifName.categoryPart(itemName)
            if ((categoryName not in cFormatList) or (categoryName not in cDefList)):
                ofh.write("+WARN PdbxCategoryStyleBase.checkStyleDefinition - missing suppressed category %s\n" % categoryName)
                missing += 1

            if itemName not in [itTup[0] for itTup in self.__cD[categoryName]]:
                ofh.write("+WARN PdbxCategoryStyleBase.checkStyleDefinition - missing suppressed item %s\n" % itemName)
                missing += 1

        return (missing == 0)

    def getCategoryList(self):
        """  Return an ordered list of categories.
        """
        try:
            return [tp[0] for tp in self.__cL]
        except Exception as e:
            print(self.__cL)
            return []

    def getItemNameList(self, categoryName):
        """
        """
        try:
            return [tp[0] for tp in self.__cD[categoryName]]
        except Exception as e:
            return []

    def getAttributeNameList(self, categoryName):
        """
        """
        try:
            return [CifName.attributePart(tp[0]) for tp in self.__cD[categoryName]]
        except Exception as e:
            return []

    def isItemExcluded(self, itemName):
        return (itemName in self.__excludeList)

    def isItemSuppressed(self, itemName):
        return (itemName in self.__suppressList)

    def isAttributeExcluded(self, categoryName, attributeName):
        return (CifName.itemName(categoryName, attribute) in self.__excludeList)

    def isAttributeSuppressed(self, categoryName, attributeName):
        return (CifName.itemName(categoryName, attribute) in self.__suppressList)

    def getItemNameAndDefaultList(self, categoryName):
        """  Return an ordered list of (itemName,defaultValue) tuples for the input category.
        """
        try:
            return [(tp[0], tp[3]) for tp in self.__cD[categoryName]]
        except Exception as e:
            return []

    def getItemNameTypeAndDefaultList(self, categoryName):
        """  Return an ordered list of (itemName,defaultValue) tuples for the input category.
        """
        try:
            return [(tp[0], tp[2], tp[3]) for tp in self.__cD[categoryName]]
        except Exception as e:
            return []



##
# File: FilterCif.py
# Date: 2016-05-05
#
# Updates:
##
"""Methods for filtering an mmCIF to remove individual items and
categories based on a parsed file already in memory from an IoAdapter
"""

import sys
import copy
import logging
logger = logging.getLogger(__name__)


from mmcif.api.PdbxContainers import CifName
from mmcif.api.DataCategory import DataCategory


class FilterCif(object):
    """ Cif filtering tool"""

    def __init__(self, verbose=False,
                 log=sys.stderr):
        """
        :param 'verbose': boolean flag to activate verbose logging
        :param 'log': open stream for logging
        :param 'IoAdapter' : Adapater for I/O of dictionary
        """
        self.__verbose = verbose
        self.__debug = True
        self.__lfh = log
        # Dictionaries indexed by mode
        self.__allCategories = {}
        self.__substituteItems = {}
        self.__excludeItems = {}
        self.__excludeCategories = {}
        self.__filterTags = []

    def setFilterSubstitutionDict(self, substDL):
        """Given a dictionary category describing the substitution, internalises it"""

        try:
            for sF in substDL:
                if (('item_name' in sF) and ('value' in sF) and ('mode' in sF) and ('tag' in sF)):
                    mode = sF['mode']
                    itemName = sF['item_name']
                    tag = sF['tag']
                    value = sF['value']
                    cat = CifName.categoryPart(itemName)
                    attr = CifName.attributePart(itemName)

                    self.__substituteItems.setdefault(mode, {})
                    self.__substituteItems[mode].setdefault(cat, {})
                    self.__substituteItems[mode][cat][attr] = [tag, value]

                    # Update list of categories to operate on
                    self.__allCategories.setdefault(mode, set())
                    self.__allCategories[mode].add(cat)
            return True
        except Exception as e:
            logger.exception("Failing with %s" % str(e))

        return False

    def setFilterItemExclusionDict(self, excludeDL):
        """Given a dictionary category describing the substitution, internalises it"""

        try:
            for eI in excludeDL:
                if (('item_id' in eI) and ('mode' in eI) and ('tag' in eI)):
                    mode = eI['mode']
                    itemId = eI['item_id']
                    tag = eI['tag']
                    cat = CifName.categoryPart(itemId)
                    attr = CifName.attributePart(itemId)

                    self.__excludeItems.setdefault(mode, {})
                    self.__excludeItems[mode].setdefault(cat, {})
                    self.__excludeItems[mode][cat][attr] = tag

                    # Update list of categories to operate on
                    self.__allCategories.setdefault(mode, set())
                    self.__allCategories[mode].add(cat)
            return True
        except Exception as e:
            logger.exception("Failing with %s" % str(e))

    def setFilterCategoryExclusionDict(self, excludeDL):
        """Given a dictionary category describing the substitution, internalises it"""

        try:
            for eI in excludeDL:
                if (('category_id' in eI) and ('mode' in eI) and ('tag' in eI)):
                    mode = eI['mode']
                    catId = eI['category_id']
                    tag = eI['tag']

                    # Removes the underscore
                    cat = CifName.categoryPart(catId)
                    self.__excludeCategories.setdefault(mode, {})
                    self.__excludeCategories[mode].setdefault(cat, {})
                    self.__excludeCategories[mode][cat] = tag

                    # Update list of categories to operate on
                    self.__allCategories.setdefault(mode, set())
                    self.__allCategories[mode].add(cat)
            return True
        except Exception as e:
            logger.exception("Failing with %s" % str(e))

    def setFilterTagList(self, tagList=[]):
        """Sets the filter context for filtering"""
        if not tagList or None in tagList:
            tagList = []
        self.__filterTags = tagList

    def filter(self, inpContainer, outContainer, mode):
        """Filters the object in memory and returns a new one.   Note - in memory objects may be updated due to data structure sharing"""

        allCats = self.__allCategories.get(mode, [])
        substItems = self.__substituteItems.get(mode, {})
        excludeItems = self.__excludeItems.get(mode, {})
        excludeCats = self.__excludeCategories.get(mode, {})

        for inpObjName in inpContainer.getObjNameList():
            srcCatObj = inpContainer.getObj(inpObjName)

            if inpObjName not in allCats:
                # Shortcut - not in an effected category
                outContainer.append(srcCatObj)
                continue

            # Check filtering of category
            catName = srcCatObj.getName()

            if excludeCats.get(catName, None) in self.__filterTags:
                # Exclude category from output - skip
                continue

            copyCategory = False

            # Check if removing an attribute
            if catName in excludeItems:
                filterAttrs = []
                for item, tag in excludeItems[catName].items():
                    if tag in self.__filterTags:
                        filterAttrs.append(item)

                # Are there attributes to filter?
                if len(filterAttrs) != 0:
                    if len(filterAttrs) == len(srcCatObj.getAttributeList()):
                        # Category will be empty - skip copy
                        continue

                    if not copyCategory:
                        dstCatObj = self.__copyCatObj(srcCatObj)
                        copyCategory = True

                    for attr in filterAttrs:
                        dstCatObj.removeAttribute(attr)

            # Check if substitutions needed
            if catName in substItems:
                for item, tagTup in substItems[catName].items():
                    tag = tagTup[0]
                    value = tagTup[1]
                    if tag in self.__filterTags and srcCatObj.hasAttribute(item):
                        if not copyCategory:
                            dstCatObj = self.__copyCatObj(srcCatObj)
                            copyCategory = True

                        # Check again - as we may have tossed the attribute in
                        # item filtering
                        if dstCatObj.hasAttribute(item):
                            for row in range(dstCatObj.getRowCount()):
                                dstCatObj.setValue(value, item, row)

            # Copy data to output
            if copyCategory:
                outContainer.append(dstCatObj)
            else:
                outContainer.append(srcCatObj)

        return True

    def __copyCatObj(self, srcCatObj):
        """Copyies the data contents so original left intact"""
        outCatObj = DataCategory(srcCatObj.getName(),
                                 list(srcCatObj.getAttributeList()),
                                 copy.deepcopy(srcCatObj.getRowList()))

        return outCatObj



##
# File: PdbxStatusHistoryCategoryStyle.py
# Date: 7-July-2014  J. Westbrook
#
# Updates:
#     11-Dec-2014  jdw  make status --> status_begin/status_end
#      6-Jan-2015  jdw  add pdb_id
#      5-Mar-2018  jdw Py2-Py3 and refactor for Python Packaging
##
"""
Abbreviated PDBx style defining data categories containing status history  details.

"""
from __future__ import absolute_import
__docformat__ = "restructuredtext en"
__author__ = "John Westbrook"
__email__ = "john.westbrook@rcsb.org"
__license__ = "Apache 2.0"
__version__ = "V0.01"


from mmcif_utils.style.PdbxCategoryStyleBase import PdbxCategoryStyleBase


class PdbxStatusHistoryCategoryStyle(PdbxCategoryStyleBase):
    _styleId = "PDBX_STATUS_HISTORY_V1"
    _categoryInfo = [
        ('pdbx_database_status_history', 'table'),
    ]
    _cDict = {
        'pdbx_database_status_history': [
            ('_pdbx_database_status_history.entry_id', '%s', 'str', ''),
            ('_pdbx_database_status_history.pdb_id', '%s', 'str', ''),
            ('_pdbx_database_status_history.ordinal', '%s', 'str', ''),
            ('_pdbx_database_status_history.date_begin', '%s', 'str', ''),
            ('_pdbx_database_status_history.date_end', '%s', 'str', ''),
            ('_pdbx_database_status_history.status_code_begin', '%s', 'str', ''),
            ('_pdbx_database_status_history.status_code_end', '%s', 'str', ''),
            ('_pdbx_database_status_history.annotator', '%s', 'str', ''),
            ('_pdbx_database_status_history.details', '%s', 'str', ''),
            ('_pdbx_database_status_history.delta_days', '%.4f', 'str', '')
        ]
    }

    _excludeList = []
    _suppressList = []
    #

    def __init__(self):
        super(PdbxStatusHistoryCategoryStyle, self).__init__(styleId=PdbxStatusHistoryCategoryStyle._styleId,
                                                             catFormatL=PdbxStatusHistoryCategoryStyle._categoryInfo,
                                                             catItemD=PdbxStatusHistoryCategoryStyle._cDict,
                                                             excludeList=PdbxStatusHistoryCategoryStyle._excludeList,
                                                             suppressList=PdbxStatusHistoryCategoryStyle._suppressList)



##
# File: PdbxPersist.py
# Date: 9-Jan-2012  John Westbrook
#
# Updates:
# 20-Jan-2012  jdw refactor and simplify persisted data - using only primitive data types.
# 24-Jan-2012  jdw added code but did not implement some methods for locking if needed in future.
#
# 17-Feb-2012  jdw added convenience methods updating individual objects in stores -
#                 updateOneObject() -
#
# 20-Feb-2012  jdw adjustments to nomenclature in fetch methods.
# 21-Feb-2012  jdw Introduce lock file support within atomic store/fetch/update/... methods.
#                  This is also implemented more coarsely for open/close methods.
# 22-Feb-2012  jdw add move method with destination locking and reindexing.
# 24-Feb-2012  jdw add method to recover index store
##
"""
Class supporting Python dictionary level (by Pdbx category) persistence.

"""
__docformat__ = "restructuredtext en"
__author__ = "John Westbrook"
__email__ = "john.westbrook@rcsb.org"
__license__ = "Apache 2.0"
__version__ = "V0.01"


import sys
import shelve
import shutil
import pickle
import logging
logger = logging.getLogger(__name__)

from mmcif.api.PdbxContainers import DefinitionContainer, DataContainer
from mmcif.api.DataCategory import DataCategory
from mmcif_utils.persist.LockFile import LockFile


class PdbxPersist(object):
    """ Persistent storage for instances of PDBx/mmCIF category objects.

    """

    def __init__(self, verbose=True, log=sys.stderr):
        self.__verbose = verbose
        self.__debug = False
        self.__lfh = log
        #
        self.__inputPdbxfilePath = None
        self.__containerList = []

        self.__containerNameList = []
        self.__containerTypeList = []
        self.__storeObjectIndex = False
        #
        # Parameters to tune lock file management --
        self.__timeoutSeconds = 10
        self.__retrySeconds = 0.2
        # placeholder for LockFile object for open/close methods
        self.__lockObj = None
        self.__isExpired = True
        #

    def __reIndex(self):
        """ Rebuild name and type lists from container object list.
        """
        #
        self.__containerNameList = []
        self.__containerTypeList = []
        for container in self.__containerList:
            self.__containerNameList.append(container.getName())
            self.__containerTypeList.append(container.getType())
        #
        self.__isExpired = False

    def __getContainerIndex(self, name):
        try:
            if (self.__isExpired):
                self.__reIndex()
            return self.__containerNameList.index(name)
        except Exception as e:
            return None

    def setContainerList(self, containerList=None):
        """ Initialize container data in the internal containlist to be persisted to the data store.
        """
        try:
            self.__containerList = containerList
            self.__isExpired = True
            if (self.__debug):
                logger.info("+PdbxPersist.setContainerList() - Container list length %d\n" % len(self.__containerList))
            return True
        except Exception as e:
            if (self.__verbose):
                logger.info("+ERROR- PdbxPersist.setContainerList() initialization failed\n")
            if (self.__debug):
                logger.exception("Failing with %s" % str(e))
            return False

    def appendContainerList(self, containerList=None):
        """ Append container data to the internal containerlist to be persisted to the data store.
        """
        try:
            self.__containerList.extend(containerList)
            self.__isExpired = True
            if (self.__debug):
                logger.info("+PdbxPersist.appendContainerList() - Container list length %d\n" % len(self.__containerList))
            return True
        except Exception as e:
            if (self.__verbose):
                logger.info("+ERROR- PdbxPersist.appendContainerList() initialization failed\n")
            if (self.__debug):
                logger.exception("Failing with %s" % str(e))
            return False

    def getContainer(self, containerName):
        """ Return the container from internal containerlist corresponding to the input container name.

            **Note that this method is NOT directly fetching data from the internal store.  First recover the
            the store to internal storage before calling this method.
        """
        idx = self.__getContainerIndex(containerName)
        if idx is not None:
            return self.__containerList[idx]
        else:
            return None

    def getContainerList(self):
        """ Return the current container list.
        """
        return self.__containerList

    def getContainerNameList(self):
        """ Return the current list of container names.
        """
        if (self.__isExpired):
            self.__reIndex()
        return self.__containerNameList

    #
    def open(self, dbFileName="my.db", flag='r'):
        """ Open the persistent store with the input database file name and access mode.

            flag = 'r read-only'
                   'c/n new/create'
                   'w read/write'
        """
        try:
            self.__lockObj = LockFile(dbFileName, timeoutSeconds=self.__timeoutSeconds, retrySeconds=self.__retrySeconds,
                                      verbose=self.__verbose, log=self.__lfh)
            self.__lockObj.acquire()
            self.__db = shelve.open(dbFileName, flag=flag, protocol=pickle.HIGHEST_PROTOCOL)
            return True
        except Exception as e:
            if self.__lockObj is not None:
                self.__lockObj.release()
            if (self.__verbose):
                logger.info("+ERROR- PdbxPersist.open() write failed for file %s\n" % dbFileName)
            if (self.__debug):
                logger.exception("Failing with %s" % str(e))
            return False

    def getStoreContainerIndex(self):
        try:
            return self.__db['__index__']
        except Exception as e:
            return []

    def close(self):
        """ Close the persistent store
        """
        try:
            self.__db.shelve.close()
            if self.__lockObj is not None:
                self.__lockObj.release()
            return True
        except Exception as e:
            if self.__lockObj is not None:
                self.__lockObj.release()
            return False

    def moveStore(self, srcDbFilePath, dstDbFilePath):
        """ Move source store to destination store with destination locking.

            No locking is performed on the source file.
        """
        with LockFile(dstDbFilePath, timeoutSeconds=self.__timeoutSeconds, retrySeconds=self.__retrySeconds, verbose=self.__verbose, log=self.__lfh) as lf:
            retVal = self.__moveStore(srcDbFilePath, dstDbFilePath)
        return retVal

    def store(self, dbFileName="my.db"):
        """ Store the current container list in persistent database.
        """
        with LockFile(dbFileName, timeoutSeconds=self.__timeoutSeconds, retrySeconds=self.__retrySeconds, verbose=self.__verbose, log=self.__lfh) as lf:
            retVal = self.__storeShelve(dbFileName)
        return retVal

    def recover(self, dbFileName="my.db"):
        """ Recover the stored state to the current in-memory container representation.
        """
        with LockFile(dbFileName, timeoutSeconds=self.__timeoutSeconds, retrySeconds=self.__retrySeconds, verbose=self.__verbose, log=self.__lfh) as lf:
            retVal = self.__recoverShelve(dbFileName)
        return retVal

    def getIndex(self, dbFileName="my.db"):
        """ Recover the index of the persistent store.
        """
        with LockFile(dbFileName, timeoutSeconds=self.__timeoutSeconds, retrySeconds=self.__retrySeconds, verbose=self.__verbose, log=self.__lfh) as lf:
            retVal = self.__indexShelve(dbFileName)
        return retVal

    def updateOneObject(self, inputObject, dbFileName="my.db", containerName=None, containerType='data'):
        """ Update or append an object to a container in an existing store.
        """
        with LockFile(dbFileName, timeoutSeconds=self.__timeoutSeconds, retrySeconds=self.__retrySeconds, verbose=self.__verbose, log=self.__lfh) as lf:
            retVal = self.__updateObjectShelve(dbFileName=dbFileName, containerName=containerName, inputObject=inputObject, containerType=containerType)
        return retVal

    def updateContainerList(self, dbFileName="my.db", containerList=None):
        """ Update or append the contents of the input container list to an existing store.
        """
        with LockFile(dbFileName, timeoutSeconds=self.__timeoutSeconds, retrySeconds=self.__retrySeconds, verbose=self.__verbose, log=self.__lfh) as lf:
            retVal = self.__updateContainerListShelve(dbFileName=dbFileName, updateContainerList=containerList)
        return retVal

    def fetchOneObject(self, dbFileName="my.db", containerName=None, objectName=None):
        """  Fetch a single object from a named container.  This is atomic operation with respect to the store.
        """
        with LockFile(dbFileName, timeoutSeconds=self.__timeoutSeconds, retrySeconds=self.__retrySeconds, verbose=self.__verbose, log=self.__lfh) as lf:
            retVal = self.__fetchOneObjectShelve(dbFileName, containerName, objectName)
        return retVal

    def fetchObject(self, containerName=None, objectName=None):
        """ Fetch object from container from an open store.

            Use this method to extract multiple objects from a store.
        """
        return self.__fetchObjectShelve(containerName, objectName)

    def __moveStore(self, src, dst):
        """  Internal method to perform file move.
        """
        try:
            shutil.move(src, dst)
            return True
        except Exception as e:
            if (self.__verbose):
                logger.info("+ERROR- PdbxPersist.__moveStore() move failed for file %s\n" % src)
            if (self.__debug):
                logger.exception("Failing with %s" % str(e))
            return False

    def __storeShelve(self, dbFileName="my.db"):
        """  Create a new persistent store using the internal object list within the current container list.
        """
        try:
            if (self.__isExpired):
                self.__reIndex()
            db = shelve.open(dbFileName, flag='c', protocol=pickle.HIGHEST_PROTOCOL)
            db['__index__'] = self.__containerNameList
            db['__types__'] = self.__containerTypeList
            if (self.__debug):
                logger.info("+PdbxPersist.__storeShelve() - Container list length  %d\n" % len(self.__containerList))
            for ii, container in enumerate(self.__containerList):
                objNameList = container.getObjNameList()
                #
                # store the object name index for this container.
                #
                containerName = container.getName()
                ky = containerName + "||__index__"
                db[ky] = objNameList
                #
                for objName in objNameList:
                    d = {}
                    d['name'], d['aL'], d['rL'] = container.getObj(objName).get()
                    ky = containerName + "||" + objName
                    db[ky] = d
            db.close()
            return True
        except Exception as e:
            if (self.__verbose):
                logger.info("+ERROR- PdbxPersist.__storeShelve() shelve store failed for file %s\n" % dbFileName)
            if (self.__debug):
                logger.exception("Failing with %s" % str(e))
            return False

    def __indexShelve(self, dbFileName="my.db"):
        """  Recover the index of containers and objects in the persistent store.
        """
        try:
            indexD = {}
            db = shelve.open(dbFileName, flag='r', protocol=pickle.HIGHEST_PROTOCOL)
            containerNameList = db['__index__']
            containerTypeList = db['__types__']
            indexD['__containers__'] = []
            for ii, containerName in enumerate(containerNameList):
                #
                indexD['__containers__'].append((containerName, containerTypeList[ii]))
                #
                ky = containerName + "||__index__"
                objNameList = db[ky]
                #
                indexD[containerName] = []
                for objName in objNameList:
                    indexD[containerName].append(objName)

            db.close()
            return indexD
        except Exception as e:
            if (self.__verbose):
                logger.info("+ERROR- PdbxPersist.__indexShelve() shelve index failed for file %s\n" % dbFileName)
            if (self.__debug):
                logger.exception("Failing with %s" % str(e))
            return {}

    def __recoverShelve(self, dbFileName="my.db"):
        """  Recover the list of containers and their object contents from the persistent store.
        """
        try:
            self.__containerList = []
            db = shelve.open(dbFileName, flag='r', protocol=pickle.HIGHEST_PROTOCOL)
            self.__containerNameList = db['__index__']
            self.__containerTypeList = db['__types__']
            if (self.__debug):
                logger.info("+PdbxPersist.__recoverShelve() - Container name list %r\n" % self.__containerNameList)
                logger.info("+PdbxPersist.__recoverShelve() - Container type list %r\n" % self.__containerTypeList)

            for ii, containerName in enumerate(self.__containerNameList):
                if self.__containerTypeList[ii] == 'data':
                    dC = DataContainer(containerName)
                else:
                    dC = DefinitionContainer(containerName)
                #
                ky = containerName + "||__index__"
                objNameList = db[ky]

                for objName in objNameList:
                    ky = containerName + "||" + objName
                    d = db[ky]
                    dObj = DataCategory(name=d['name'], attributeNameList=d['aL'], rowList=d['rL'])
                    dC.append(dObj)
                self.__containerList.append(dC)

            db.close()
            return True
        except Exception as e:
            if (self.__verbose):
                logger.info("+ERROR- PdbxPersist.__recoverShelve() shelve recover failed for file %s\n" % dbFileName)
            if (self.__debug):
                logger.exception("Failing with %s" % str(e))
            return False

    def __fetchOneObjectShelve(self, dbFileName="my.db", containerName=None, objectName=None):
        """ Recover the object from from persistent store corresponding to the
            input container and object name.
        """
        try:
            #
            db = shelve.open(dbFileName, flag='r', protocol=pickle.HIGHEST_PROTOCOL)
            ky = containerName + '||' + objectName
            d = db[ky]
            outObject = DataCategory(name=d['name'], attributeNameList=d['aL'], rowList=d['rL'])
            db.close()
            return outObject
        except Exception as e:
            if (self.__verbose):
                logger.info("+ERROR- PdbxPersist.__fetchOneObjectShelve() shelve fetch failed for file %s %s %s \n"
                            % (dbFileName, containerName, objectName))
            if (self.__debug):
                logger.exception("Failing with %s" % str(e))
            return None

    def __fetchObjectShelve(self, containerName=None, objectName=None):
        """ Recover the object from from persistent store corresponding to the
            input container and object name.

            shelve store must be opened from prior call.
        """
        try:
            #
            ky = containerName + '||' + objectName
            d = self.__db[ky]
            outObject = DataCategory(name=d['name'], attributeNameList=d['aL'], rowList=d['rL'])
            return outObject
        except Exception as e:
            if (self.__debug):
                logger.info("+ERROR- PdbxPersist.__fetchObjectShelve() shelve fetch failed %s %s \n"
                            % (containerName, objectName))
            if (self.__debug):
                logger.exception("Failing with %s" % str(e))
            #
            return None

    def __updateObjectShelve(self, dbFileName="my.db", containerName=None, inputObject=None, containerType='data'):
        """ Update/append the single input object into the named container in the persistent store.
        """
        try:
            objectName = inputObject.getName()
            #
            db = shelve.open(dbFileName, flag='w', protocol=pickle.HIGHEST_PROTOCOL)
            #
            # get container index -
            #
            containerNameList = db['__index__']
            containerTypeList = db['__types__']
            #
            # Modify or append input container --
            #
            cExists = True
            if containerName in containerNameList:
                idx = containerNameList.index(containerName)
                containerTypeList[idx] = containerType
            else:
                cExists = False
                containerNameList.append(containerName)
                containerTypeList.append(containerType)
            #
            # store updated container index lists -
            #
            db['__index__'] = containerNameList
            db['__types__'] = containerTypeList
            #
            # ------------------------------------------
            # Get object index for the target container
            #   store updates if needed --
            #
            ky = containerName + "||__index__"
            if (cExists):
                objectNameList = db[ky]
            else:
                objectNameList = []

            oExists = True
            if objectName in objectNameList:
                idx = objectNameList.index(objectName)
            else:
                oExists = False
                objectNameList.append(objectName)
                db[ky] = objectNameList

            # ---------------------------------
            #    Update the object contents --
            #
            ky = containerName + "||" + objectName

            d = {}
            d['name'], d['aL'], d['rL'] = inputObject.get()
            db[ky] = d
            db.close()
            return True
        except Exception as e:
            if (self.__verbose):
                logger.info("+ERROR- PdbxPersist.__updateObjectShelve() update failed for file %s %s \n" % (dbFileName, containerName))
            if (self.__debug):
                logger.exception("Failing with %s" % str(e))
            return False

    def __updateContainerListShelve(self, dbFileName="my.db", updateContainerList=None):
        """ Update/append the contents of the input container list into an existing persistent store.
        """
        try:
            db = shelve.open(dbFileName, flag='w', protocol=pickle.HIGHEST_PROTOCOL)
            #
            # get container index -
            #
            containerNameList = db['__index__']
            containerTypeList = db['__types__']

            for updateContainer in updateContainerList:
                #
                updateContainerName = updateContainer.getName()
                updateContainerType = updateContainer.getType()
                #
                # Modify or append input container --
                #
                cExists = True
                if updateContainerName in containerNameList:
                    idx = containerNameList.index(updateContainerName)
                    containerTypeList[idx] = updateContainerType
                else:
                    cExists = False
                    containerNameList.append(updateContainerName)
                    containerTypeList.append(updateContainerType)
                #
                # Store updated container index lists -
                #
                db['__index__'] = containerNameList
                db['__types__'] = containerTypeList

                #
                # ------------------------------------------
                # Get object index for the target container
                #   and store updates if needed --
                #
                idxKy = updateContainerName + "||__index__"
                if (cExists):
                    objectNameList = db[idxKy]
                else:
                    objectNameList = []

                #
                updateObjectNameList = updateContainer.getObjNameList()
                #
                for updateObjectName in updateObjectNameList:
                    updateObject = updateContainer.getObj(updateObjectName)
                    oExists = True
                    if updateObjectName not in objectNameList:
                        oExists = False
                        objectNameList.append(updateObjectName)
                        db[idxKy] = objectNameList

                    # ---------------------------------
                    #   Now update the object contents --
                    #
                    ky = updateContainerName + "||" + updateObjectName
                    d = {}
                    d['name'], d['aL'], d['rL'] = updateObject.get()
                    db[ky] = d
            #
            db.close()
            return True
        except Exception as e:
            if (self.__verbose):
                logger.info("+ERROR- PdbxPersist.__updateContainerListShelve() update failed for store %s\n" % dbFileName)
            if (self.__debug):
                logger.exception("Failing with %s" % str(e))
            return False

    def __attributePart(self, name):
        i = name.find(".")
        if i == -1:
            return None
        else:
            return name[i + 1:]

    ##
    # Obsolete methods -- which serial objects as well as data -- found to be less efficient.
    ##
    def __fetchOneObjectShelve2(self, dbFileName="my.db", containerName=None, objectName=None):
        """ Recover the object from from persistent store corresponding to the
            input container and object name.
        """
        try:
            #
            db = shelve.open(dbFileName, flag='r', protocol=pickle.HIGHEST_PROTOCOL)
            ky = containerName + '||' + objectName
            outContainer = db[ky]
            db.close()
            return outContainer
        except Exception as e:
            if (self.__verbose):
                logger.info("+ERROR- PdbxPersist.__fetchOneObjectShelve() shelve fetch failed for file %s %s %s \n"
                            % (dbFileName, containerName, objectName))
            if (self.__debug):
                logger.exception("Failing with %s" % str(e))
            return None

    def __storeShelve2(self, dbFileName="my.db"):
        """  Create a new store the object list in the current container list into the persistent store.
        """

        try:
            db = shelve.open(dbFileName, flag='c', protocol=pickle.HIGHEST_PROTOCOL)
            # print self.__containerNameList
            db['__index__'] = self.__containerNameList
            db['__types__'] = self.__containerTypeList
            for ii, container in enumerate(self.__containerList):
                objNameList = container.getObjNameList()
                #
                # store the object name index for this container.
                #
                containerName = container.getName()
                ky = containerName + "||__index__"
                db[ky] = objNameList
                #
                for objName in objNameList:
                    dObj = container.getObj(objName)
                    ky = containerName + "||" + objName
                    db[ky] = dObj
            db.close()
            return True
        except Exception as e:
            if (self.__verbose):
                logger.info("+ERROR- PdbxPersist.__storeShelve() shelve store failed for file %s\n" % dbFileName)
            if (self.__debug):
                logger.exception("Failing with %s" % str(e))
            return False

    def __recoverShelve2(self, dbFileName="my.db"):
        """  Recover the list of containers and their object contents from the persistent store.
        """
        try:
            self.__containerList = []
            db = shelve.open(dbFileName, flag='r', protocol=pickle.HIGHEST_PROTOCOL)
            self.__containerNameList = db['__index__']
            self.__containerTypeList = db['__types__']
            for ii, containerName in enumerate(self.__containerNameList):
                if self.__containerTypeList[ii] == 'data':
                    dC = DataContainer(containerName)
                else:
                    dC = DefinitionContainer(containerName)
                #
                ky = containerName + "||__index__"
                objNameList = db[ky]

                for objName in objNameList:
                    ky = containerName + "||" + objName
                    dObj = db[ky]
                    dC.append(dObj)
                self.__containerList.append(dC)

            db.close()
            return True
        except Exception as e:
            if (self.__verbose):
                logger.info("+ERROR- PdbxPersist.__recoverShelve() shelve recover failed for file %s\n" % dbFileName)
            if (self.__debug):
                logger.exception("Failing with %s" % str(e))
            return False

    def __updateObjectShelve2(self, dbFileName="my.db", containerName=None, inputObject=None, containerType='data'):
        """ Update/append the input object into the named container in the persistent store.
        """
        try:
            objectName = inputObject.getName()
            #
            db = shelve.open(dbFileName, flag='w', protocol=pickle.HIGHEST_PROTOCOL)
            #
            # get container index -
            #
            containerNameList = db['__index__']
            containerTypeList = db['__types__']
            #
            # Modifying or appending containers --
            #
            cExists = True
            if containerName in containerNameList:
                idx = containerNameList.index(containerName)
                containerTypeList[idx] = containerType
            else:
                cExists = False
                containerNameList.append(containerName)
                containerTypeList.append(containerType)
            #
            # store updated container index lists -
            #
            db['__index__'] = containerNameList
            db['__types__'] = containerTypeList
            #
            # ------------------------------------------
            # Get object index for the target container
            #   store updates if needed --
            #
            ky = containerName + "||__index__"
            if (cExists):
                objectNameList = db[ky]
            else:
                objectNameList = []

            oExists = True
            if objectName in objectNameList:
                idx = objectNameList.index(objectName)
            else:
                oExists = False
                objectNameList.append(objectName)
                db[ky] = objectNameList

            # ---------------------------------
            #    Update the object contents --
            #
            ky = containerName + "||" + objectName
            db[ky] = inputObject
            db.close()
            return True
        except Exception as e:
            if (self.__verbose):
                logger.info("+ERROR- PdbxPersist.__updateObjectShelve() update failed for file %s %s \n" % (dbFileName, containerName))
            if (self.__debug):
                logger.exception("Failing with %s" % str(e))
            return False



##
# File: PdbxChemCompModel.py
# Date: 29-Nov-2014
#
# Update:
#
##
"""
A collection of accessor classes supporting chemical component model data categories.

"""
__docformat__ = "restructuredtext en"
__author__ = "John Westbrook"
__email__ = "john.westbrook@rcsb.org"
__license__ = "Apache 2.0"
__version__ = "V0.01"


import sys

import logging
logger = logging.getLogger(__name__)

from mmcif_utils.chemcomp.PdbxChemComp import PdbxChemCompConstants


class PdbxChemCompModel(object):
    ''' Accessor methods chemical component model attributes.

    '''

    def __init__(self, compD, verbose=True, log=sys.stderr):
        self.__verbose = verbose
        self.__debug = False
        self.__lfh = log
        # Attribute mapping of chem_comp data category to dictionary structure (e.g. _category.attribute)
        self.__atD = compD

    def __getAttribute(self, name):
        try:
            return self.__atD[name]
        except Exception as e:
            return None

    def getId(self):
        return self.__getAttribute('id')

    def getCompId(self):
        return self.__getAttribute('comp_id')


class PdbxChemCompModelAtom(object):
    ''' Accessor methods chemical component model atom attributes.

    '''

    def __init__(self, atomD, verbose=True, log=sys.stderr):
        self.__verbose = verbose
        self.__debug = False
        self.__lfh = log
        # Attribute mapping of chem_comp_atom data category to dictionary structure (e.g. _category.attribute)
        self.__atD = atomD

    def __getAttribute(self, name):
        try:
            return self.__atD[name]
        except Exception as e:
            return None

    def getName(self):
        return self.__getAttribute('atom_id')

    def getType(self):
        return self.__getAttribute('type_symbol')

    def getAtNo(self):
        try:
            tyU = str(self.getType()).upper()
            if ((tyU == 'D') or (tyU == 'T')):
                tyU = 'H'
            return PdbxChemCompConstants._periodicTable.index(tyU) + 1
        except Exception as e:
            logger.exception("Failing with %s" % str(e))
            return 0

    def getIsotope(self):
        ty = self.getType()
        if ty == 'D':
            return 2
        elif ty == 'T':
            return 3
        else:
            return 0

    def getFormalCharge(self):
        try:
            return int(self.__getAttribute('charge'))
        except Exception as e:
            return 0

    def hasModelCoordinates(self):
        x = self.__getAttribute('model_Cartn_x')
        y = self.__getAttribute('model_Cartn_y')
        z = self.__getAttribute('model_Cartn_z')
        #
        return ((x is not None) and (y is not None) and (z is not None))

    def getModelCoordinates(self):
        """ Returns (x,y,z)
        """
        try:
            x = float(self.__getAttribute('model_Cartn_x'))
            y = float(self.__getAttribute('model_Cartn_y'))
            z = float(self.__getAttribute('model_Cartn_z'))
            return (x, y, z)
        except Exception as e:
            return (None, None, None)


class PdbxChemCompModelBond(object):
    ''' Accessor methods chemical component model bond attributes.

    '''

    def __init__(self, bondD, verbose=True, log=sys.stderr):
        self.__verbose = verbose
        self.__debug = False
        self.__lfh = log
        # Attribute mapping of chem_comp_bond data category to dictionary structure (e.g. _category.attribute)
        self.__atD = bondD

    def __getAttribute(self, name):
        try:
            return self.__atD[name]
        except Exception as e:
            return None

    def getBond(self):
        """ Returns (atomI,atomJ) atom ids from the atom list.
        """
        return (self.__getAttribute('atom_id_1'), self.__getAttribute('atom_id_2'))

    def getType(self):
        return self.__getAttribute('value_order')

    def getIntegerType(self):
        bT = self.__getAttribute('value_order')
        if bT == 'SING':
            return 1
        elif bT == 'DOUB':
            return 2
        elif bT == 'TRIP':
            return 3
        elif bT == 'QUAD':
            return 4
        else:
            return 0


class PdbxChemCompModelDescriptor(object):
    ''' Accessor methods chemical component model descriptor.

    '''

    def __init__(self, dD, verbose=True, log=sys.stderr):
        self.__verbose = verbose
        self.__debug = False
        self.__lfh = log
        # Attribute mapping of chem_comp_bond data category to dictionary structure (e.g. _category.attribute)
        self.__atD = dD

    def __getAttribute(self, name):
        try:
            return self.__atD[name]
        except Exception as e:
            return None

    def getType(self):
        return self.__getAttribute('type')

    def getDescriptor(self):
        return self.__getAttribute('descriptor')


class PdbxChemCompModelFeature(object):
    ''' Accessor methods chemical component model features.

    '''

    def __init__(self, fD, verbose=True, log=sys.stderr):
        self.__verbose = verbose
        self.__debug = False
        self.__lfh = log
        # Attribute mapping of chem_comp_bond data category to dictionary structure (e.g. _category.attribute)
        self.__atD = fD

    def __getAttribute(self, name):
        try:
            return self.__atD[name]
        except Exception as e:
            return None

    def getFeatureName(self):
        return self.__getAttribute('feature_name')

    def getFeatureValue(self):
        return self.__getAttribute('feature_value')


class PdbxChemCompModelReference(object):
    ''' Accessor methods chemical component model reference.

    '''

    def __init__(self, rD, verbose=True, log=sys.stderr):
        self.__verbose = verbose
        self.__debug = False
        self.__lfh = log
        # Attribute mapping of chem_comp_bond data category to dictionary structure (e.g. _category.attribute)
        self.__atD = rD

    def __getAttribute(self, name):
        try:
            return self.__atD[name]
        except Exception as e:
            return None

    def getDatabaseName(self):
        return self.__getAttribute('database_name')

    def getDatabaseCode(self):
        return self.__getAttribute('database_code')


class PdbxChemCompModelAudit(object):
    ''' Accessor methods chemical component model audit records.

    '''

    def __init__(self, aD, verbose=True, log=sys.stderr):
        self.__verbose = verbose
        self.__debug = False
        self.__lfh = log
        # Attribute mapping of chem_comp_bond data category to dictionary structure (e.g. _category.attribute)
        self.__atD = aD

    def __getAttribute(self, name):
        try:
            return self.__atD[name]
        except Exception as e:
            return None

    def getActionType(self):
        return self.__getAttribute('action_type')

    def getDate(self):
        return self.__getAttribute('date')

    def getProcessingSite(self):
        return self.__getAttribute('processing_site')

    def getAnnotator(self):
        return self.__getAttribute('annotator')

    def getDetails(self):
        return self.__getAttribute('details')



##
# File: ChemCompCategoryStyle.py
# Date: 4-Nov-2012  John Westbrook
#
# Updates:
#  11-Feb-2013 jdw   add style identifier
#   5-Mar-2018 jdw Py2-Py3 and refactor for Python Packaging
##
"""
Standard style details for chemical component data categories in the
chemical component dictionary.

"""
from __future__ import absolute_import
__docformat__ = "restructuredtext en"
__author__ = "John Westbrook"
__email__ = "john.westbrook@rcsb.org"
__license__ = "Apache 2.0"
__version__ = "V0.01"


from mmcif_utils.style.PdbxCategoryStyleBase import PdbxCategoryStyleBase


class ChemCompCategoryStyle(PdbxCategoryStyleBase):
    #
    _styleId = "CHEM_COMP_V1"

    _categoryInfo = [('chem_comp', 'key-value'),
                     ('chem_comp_atom', 'table'),
                     ('chem_comp_bond', 'table'),
                     ('pdbx_chem_comp_identifier', 'table'),
                     ('pdbx_chem_comp_descriptor', 'table'),
                     ('pdbx_chem_comp_audit', 'table'),
                     ('pdbx_chem_comp_import', 'table'),
                     ('pdbx_chem_comp_atom_edit', 'table'),
                     ('pdbx_chem_comp_bond_edit', 'table')
                     ]
    _cDict = {
        'chem_comp': [
            ('_chem_comp.id', '%s', 'str', ''),
            ('_chem_comp.name', '%s', 'str', ''),
            ('_chem_comp.type', '%s', 'str', ''),
            ('_chem_comp.pdbx_type', '%s', 'str', ''),
            ('_chem_comp.formula', '%s', 'str', ''),
            ('_chem_comp.mon_nstd_parent_comp_id', '%s', 'str', ''),
            ('_chem_comp.pdbx_synonyms', '%s', 'str', ''),
            ('_chem_comp.pdbx_formal_charge', '%s', 'str', ''),
            ('_chem_comp.pdbx_initial_date', '%s', 'str', ''),
            ('_chem_comp.pdbx_modified_date', '%s', 'str', ''),
            ('_chem_comp.pdbx_ambiguous_flag', '%s', 'str', ''),
            ('_chem_comp.pdbx_release_status', '%s', 'str', ''),
            ('_chem_comp.pdbx_replaced_by', '%s', 'str', ''),
            ('_chem_comp.pdbx_replaces', '%s', 'str', ''),
            ('_chem_comp.formula_weight', '%s', 'str', ''),
            ('_chem_comp.one_letter_code', '%s', 'str', ''),
            ('_chem_comp.three_letter_code', '%s', 'str', ''),
            ('_chem_comp.pdbx_model_coordinates_details', '%s', 'str', ''),
            ('_chem_comp.pdbx_model_coordinates_missing_flag', '%s', 'str', ''),
            ('_chem_comp.pdbx_ideal_coordinates_details', '%s', 'str', ''),
            ('_chem_comp.pdbx_ideal_coordinates_missing_flag', '%s', 'str', ''),
            ('_chem_comp.pdbx_model_coordinates_db_code', '%s', 'str', ''),
            ('_chem_comp.pdbx_subcomponent_list', '%s', 'str', ''),
            ('_chem_comp.pdbx_processing_site', '%s', 'str', '')
        ],
        'chem_comp_atom': [
            ('_chem_comp_atom.comp_id', '%s', 'str', ''),
            ('_chem_comp_atom.atom_id', '%s', 'str', ''),
            ('_chem_comp_atom.alt_atom_id', '%s', 'str', ''),
            ('_chem_comp_atom.type_symbol', '%s', 'str', ''),
            ('_chem_comp_atom.charge', '%s', 'str', ''),
            ('_chem_comp_atom.pdbx_align', '%s', 'str', ''),
            ('_chem_comp_atom.pdbx_aromatic_flag', '%s', 'str', ''),
            ('_chem_comp_atom.pdbx_leaving_atom_flag', '%s', 'str', ''),
            ('_chem_comp_atom.pdbx_stereo_config', '%s', 'str', ''),
            ('_chem_comp_atom.model_Cartn_x', '%s', 'str', ''),
            ('_chem_comp_atom.model_Cartn_y', '%s', 'str', ''),
            ('_chem_comp_atom.model_Cartn_z', '%s', 'str', ''),
            ('_chem_comp_atom.pdbx_model_Cartn_x_ideal', '%s', 'str', ''),
            ('_chem_comp_atom.pdbx_model_Cartn_y_ideal', '%s', 'str', ''),
            ('_chem_comp_atom.pdbx_model_Cartn_z_ideal', '%s', 'str', ''),
            ('_chem_comp_atom.pdbx_component_atom_id', '%s', 'str', ''),
            ('_chem_comp_atom.pdbx_component_comp_id', '%s', 'str', ''),
            ('_chem_comp_atom.pdbx_ordinal', '%s', 'str', '')
        ],
        'chem_comp_bond': [
            ('_chem_comp_bond.comp_id', '%s', 'str', ''),
            ('_chem_comp_bond.atom_id_1', '%s', 'str', ''),
            ('_chem_comp_bond.atom_id_2', '%s', 'str', ''),
            ('_chem_comp_bond.value_order', '%s', 'str', ''),
            ('_chem_comp_bond.pdbx_aromatic_flag', '%s', 'str', ''),
            ('_chem_comp_bond.pdbx_stereo_config', '%s', 'str', ''),
            ('_chem_comp_bond.pdbx_ordinal', '%s', 'str', '')
        ],
        'pdbx_chem_comp_descriptor': [
            ('_pdbx_chem_comp_descriptor.comp_id', '%s', 'str', ''),
            ('_pdbx_chem_comp_descriptor.type', '%s', 'str', ''),
            ('_pdbx_chem_comp_descriptor.program', '%s', 'str', ''),
            ('_pdbx_chem_comp_descriptor.program_version', '%s', 'str', ''),
            ('_pdbx_chem_comp_descriptor.descriptor', '%s', 'str', '')
        ],
        'pdbx_chem_comp_identifier': [
            ('_pdbx_chem_comp_identifier.comp_id', '%s', 'str', ''),
            ('_pdbx_chem_comp_identifier.type', '%s', 'str', ''),
            ('_pdbx_chem_comp_identifier.program', '%s', 'str', ''),
            ('_pdbx_chem_comp_identifier.program_version', '%s', 'str', ''),
            ('_pdbx_chem_comp_identifier.identifier', '%s', 'str', '')
        ],
        'pdbx_chem_comp_import': [
            ('_pdbx_chem_comp_identifier.comp_id', '%s', 'str', ''),
            ('_pdbx_chem_comp_identifier.comp_alias_id', '%s', 'str', ''),
            ('_pdbx_chem_comp_identifier.model_path', '%s', 'str', '')
        ],
        'pdbx_chem_comp_audit': [
            ('_pdbx_chem_comp_audit.comp_id', '%s', 'str', ''),
            ('_pdbx_chem_comp_audit.action_type', '%s', 'str', ''),
            ('_pdbx_chem_comp_audit.date', '%s', 'str', ''),
            ('_pdbx_chem_comp_audit.processing_site', '%s', 'str', ''),
            ('_pdbx_chem_comp_audit.annotator', '%s', 'str', ''),
            ('_pdbx_chem_comp_audit.details', '%s', 'str', '')
        ],
        'pdbx_chem_comp_atom_edit': [
            ('_pdbx_chem_comp_atom_edit.ordinal', '%d', 'int', ''),
            ('_pdbx_chem_comp_atom_edit.comp_id', '%s', 'str', ''),
            ('_pdbx_chem_comp_atom_edit.atom_id', '%s', 'str', ''),
            ('_pdbx_chem_comp_atom_edit.edit_op', '%s', 'str', ''),
            ('_pdbx_chem_comp_atom_edit.edit_atom_id', '%s', 'str', ''),
            ('_pdbx_chem_comp_atom_edit.edit_atom_value', '%s', 'str', '')
        ],
        'pdbx_chem_comp_bond_edit': [
            ('_pdbx_chem_comp_bond_edit.ordinal', '%d', 'int', ''),
            ('_pdbx_chem_comp_bond_edit.comp_id_1', '%s', 'str', ''),
            ('_pdbx_chem_comp_bond_edit.atom_id_1', '%s', 'str', ''),
            ('_pdbx_chem_comp_bond_edit.comp_id_2', '%s', 'str', ''),
            ('_pdbx_chem_comp_bond_edit.atom_id_2', '%s', 'str', ''),
            ('_pdbx_chem_comp_bond_edit.edit_op', '%s', 'str', ''),
            ('_pdbx_chem_comp_bond_edit.edit_bond_value', '%s', 'str', '')
        ]
    }

    _suppressList = []
    _excludeList = ['_pdbx_chem_comp_audit.annotator', '_pdbx_chem_comp_audit.details']
    #

    def __init__(self):
        super(ChemCompCategoryStyle, self).__init__(styleId=ChemCompCategoryStyle._styleId,
                                                    catFormatL=ChemCompCategoryStyle._categoryInfo,
                                                    catItemD=ChemCompCategoryStyle._cDict,
                                                    excludeList=ChemCompCategoryStyle._excludeList,
                                                    suppressList=ChemCompCategoryStyle._suppressList)
